	/* G N O K K I
	   A Linux/Unix toolset and driver for the Nokia 3110/3810/8110 mobiles.
	   Copyright (C) Hugh Blemings, 1999  Released under the terms of 
       the GNU GPL, see file COPYING for more details.
	
	   This file:  gnokii.c

	   Mainline code for gnokii utility.  Handles command line parsing
	   and reading/writing phonebook entries. */


#include	<termios.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<unistd.h>
#include	<fcntl.h>
#include	<sys/signal.h>
#include	<sys/types.h>
#include	<sys/time.h>
#include	<string.h>

#include	"misc.h"
#include	"fbus.h"

	/* Prototypes. */
void	usage(void);
void	monitormode(void);
void	getphonebook(char *argv[]);
void	writephonebook(void);
void	fbusinit(void);

	/* Main function - handles command line args then passes to 
	   separate function accordingly. */
int	main(int argc, char *argv[])
{
		/* Handle command line arguments. */
	if (argc == 1 || strcmp(argv[1], "--help") == 0) {
		usage();
		exit (0);
	}

		/* Display version,  copyright and build information. */
	if (strcmp(argv[1], "--version") == 0) {
		fprintf(stdout, "GNOKII Version 0.2.0 Copyright (C) Hugh Blemings 1999. <hugh@vsb.com.au>\n");
		fprintf(stdout, "       Built %s %s\n", __TIME__, __DATE__);
		exit (0);
	}

		/* Enter monitor mode. */
	if (strcmp(argv[1], "--monitor") == 0) {
		monitormode();
	}

		/* Get phonebook command. */
	if (argc == 5 && strcmp(argv[1], "--getphonebook") == 0) {
		getphonebook(argv);
	}

		/* Write phonebook command. */
	if (strcmp(argv[1], "--writephonebook") == 0) {
		writephonebook();
	}

		/* Either an unknown command or wrong number of args! */
	usage();
	exit (-1);
}

	/* In monitor mode we don't do much, just initialise the fbus code
	   and set the FBUS_EnableMonitoring flag. */
void	monitormode(void)
{

	fprintf (stdout, "Monitor mode...\n");
	fprintf (stdout, "Initialising FBUS interface...\n");

		/* Initialise the code for the FBUS interface. */     
	fbusinit();
	FBUS_EnableMonitoringOutput = true;

		/* loop here indefinately - allows you to see messages from FBUS
		   code in response to unknown messages etc. */
	while (1) {
		sleep(1);
	}

	FBUS_Terminate();
	exit(0);
}

	/* Get requested range of phonebook entries and output to stdout
	   in format suitable for editing and subsequent writing back
	   to phone with writephonebook. */
void	getphonebook(char *argv[])
{
		/* Temporary entry for phonebook code. */
	FBUS_PhonebookEntry		entry;
	int						count;
	int						memory_type;
	int						start_entry;
	int						end_entry;

		/* Do generic initialisation routine. */
	fbusinit();

		/* Handle command line args that set type, start and end locations. */
	memory_type = atoi (argv[2]);
	start_entry = atoi (argv[3]);
	end_entry = atoi (argv[4]);

		/* Now retrieve the requested entries. */
	for (count = start_entry; count <= end_entry; count ++) {
		if (FBUS_GetPhonebookLocation(memory_type, count, &entry) == 0) {
			fprintf(stdout, "%d|%d|%s|%s\n", memory_type, count, entry.Name, entry.Number);
		}
		else {
			fprintf(stdout, "%d|%d|Bad location or other error!\n", memory_type, count);
		}
	}
	
	FBUS_Terminate();
	exit(0);
}

	/* Read data from stdin, parse and write to phone.  The parsing
	   is relatively crude and doesn't allow for much variation
	   from the stipulated format. */
void	writephonebook(void)
{
	FBUS_PhonebookEntry		entry;
	char					line_buffer[100];
	int						memory_type, entry_number, count, sub_count;
	char					name_buffer[40], number_buffer[60];
	int						line_count;
	int						separator_count;
	int						error_code;

		/* Initialise fbus code, enable monitoring for debugging
		   purposes... */
	fbusinit();
	FBUS_EnableMonitoringOutput = true;

		/* Initialise line count. */
	line_count = 0;

		/* Go through data from stdin. */
	while(fscanf(stdin, "%s\n", line_buffer) == 1) {
		line_count ++;

			/* Do initial parse */
		if (sscanf(line_buffer, "%d|%d|", &memory_type, &entry_number) != 2) {
			fprintf(stderr, "Format problem on line %d [%s]\n", line_count, line_buffer);
		} 
		else {
			separator_count = 0;
			count = 0;

				/* Skip first two separators. */
			while (count < strlen(line_buffer) && separator_count < 2) {
				if (line_buffer[count] == '|') {
					separator_count ++;
				}
				count ++;
			}
				/* Get next field (name) */
			sub_count = 0;	
			while ((count < strlen(line_buffer)) && (line_buffer[count] != '|')) {
				name_buffer[sub_count] = line_buffer[count];

				count ++;
				sub_count ++;
			}
			name_buffer[sub_count] = 0;
			count++;

				/* Get last field (number) */
			sub_count = 0;	
			while ((count < strlen(line_buffer)) && (line_buffer[count] != '|')) {
				number_buffer[sub_count] = line_buffer[count];

				count ++;
				sub_count ++;
			}
			number_buffer[sub_count] = 0;

		}

			/* Copy into entry. */	
		strcpy (entry.Name, name_buffer);
		strcpy (entry.Number, number_buffer);

			/* Do write and report success/failure. */
		error_code = FBUS_WritePhonebookLocation(memory_type, entry_number, &entry);

		if (error_code == FBE_NONE) {
			fprintf (stdout, "Write Succeeded: memory_type:%d, loc:%d, name: %s, number: %s\n", memory_type, entry_number, entry.Name, entry.Number);
		}
		else {
			fprintf (stdout, "Write FAILED(%d): memory_type:%d, loc:%d, name: %s, number: %s\n", error_code, memory_type, entry_number, entry.Name, entry.Number);
		}
	}

	FBUS_Terminate();
	exit(0);
}


void	usage(void)
{
	fprintf(stdout, "usage: gnokii {--help|--monitor|--version}\n");
	fprintf(stdout, "       gnokii [--getphonebook] [memory type] [start] [end]\n");
	fprintf(stdout, "       gnokii [--writephonebook]\n\n");
	

	fprintf(stdout, "          --help            display this usage/help file.\n\n");
	fprintf(stdout, "          --monitor         continually updates phone status to stderr.\n\n");
	fprintf(stdout, "          --version         displays version and copyright information.\n\n");

	fprintf(stdout, "          --getphonebook    gets phonebook entries from specified memory type\n");
 	fprintf(stdout, "                            (2=SIM, 0=Internal?) starting at entry [start] and\n");
	fprintf(stdout, "                            ending at [end].  Entries are dumped to stdout in\n");
	fprintf(stdout, "                            format suitable for editing and passing back to\n");
	fprintf(stdout, "                            writephonebook command.\n\n");

	fprintf(stdout, "          --writephonebook  reads data from stdin in and writes to phonebook.\n");
	fprintf(stdout, "                            Uses the same format as provided by the output of\n");
	fprintf(stdout, "                            the getphonebook command.\n");


}

void	fbusinit(void)
{
	int		count;

		/* Initialise the code for the FBUS interface. */     
	FBUS_Initialise();

		/* First (and important!) wait for FBUS link to be active. 
		   we allow 10 seconds... */
	count = 0;

	while (count < 200 && FBUS_LinkOK == false) {
		count ++;
		usleep (50000);
	}

	if (FBUS_LinkOK == false) {
		fprintf (stderr, "Hmmm... FBUS_LinkOK never went true.  Quitting. \n");
		exit (-1);
	}
}
